/*--------------------------------------------------------------------*\
  File:      FontMenu.cpp
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Source file containing the member functions for the
      FontMenu class.
  ID:        $Id: FontMenu.cpp,v 1.31 1998/07/20 05:16:46 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include "FontMenu.h"
#include "Arrayomatic.h"
#include "FontMenuItem.h"
#include <ctype.h>
#include <string.h>


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

// Initialize the FontMenuItem private static const data members
const uint32 FontMenu::mk_msg_type('FSel');
const char * const FontMenu::mk_msg_fam("family");
const char * const FontMenu::mk_msg_stl("style");


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-= Function Definitions =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

//====================================================================
FontMenu::FontMenu(const char * const a_title, menu_layout a_layout) :
//====================================================================
	Inherited(a_title, a_layout)
{
	Init();
}

//====================================================================
FontMenu::FontMenu(const char * const a_title, float a_width, float a_height) :
//====================================================================
	Inherited(a_title, a_width, a_height)
{
	Init();
}

//====================================================================
FontMenu::FontMenu(BMessage *a_archive) :
//====================================================================
	Inherited(a_archive)
{
	Init();
}

//====================================================================
FontMenu::~FontMenu(void)
//====================================================================
{
	;
}

//====================================================================
void FontMenu::FamilyAndStyle(font_family &a_family, font_style &a_style) const
//====================================================================
{
	strcpy(a_family, m_family);
	strcpy(a_style, m_style);
}

//====================================================================
void FontMenu::Initialize(const bool a_render_style, const bool a_render_family)
//====================================================================
{
	// Reset the family and style
	m_family[0] = '\0';
	m_style[0] = '\0';
	
	// Get rid of any old items
	BMenuItem *family_item;
	
	while ((family_item = RemoveItem(static_cast<int32>(0))) != NULL)
	{
		delete family_item;
	}
	
	// Create a BMenuItem for each family
	int32 num_families(count_font_families());
	font_family family;
	uint32 flags;
	Arrayomatic<BMenuItem *> family_array(num_families);
	
	for (int32 i(0); i < num_families; i++)
	{
		// Get the family
		if (get_font_family(i, &family, &flags) != B_NO_ERROR)
		{
			continue;
		}
		
		if ((family_item = ConstructFamilyMenuItem(family, a_render_style, a_render_family)) == NULL)
		{
			continue;
		}
		
		// Push the family BMenu onto the temporary array
		family_array.PushItem(family_item);
	}
	
	// Sort the array alphabetically (non-case-sensitive)
	family_array.Sort(MenuItemComp);
	
	// Pop each family BMenuItem off of the sorted array and at it
	// to the FontMenu
	while (family_array.PopItem(family_item) == ENOERR)
	{
		AddItem(family_item);
	}
	
	// Set the default font
	font_style style;
	be_plain_font->GetFamilyAndStyle(&family, &style);
	SetFamilyAndStyle(family, style);
}

//====================================================================
void FontMenu::MessageReceived(BMessage *a_msg)
//====================================================================
{
	if (a_msg->what == mk_msg_type)
	{
		SetFamilyAndStyle(a_msg);
	}
	else
	{
		Inherited::MessageReceived(a_msg);
	}
}

//====================================================================
void FontMenu::SetFamilyAndStyle(const BMessage *a_msg)
//====================================================================
{
	SetFamilyAndStyle(a_msg->FindString(mk_msg_fam), a_msg->FindString(mk_msg_stl));
}

//====================================================================
void FontMenu::SetFamilyAndStyle(const font_family &a_family, const font_style &a_style)
//====================================================================
{
	BMenu *menu;
	BMenuItem *menu_item;
	
	// Unmark the old family and style
	while ((menu_item = FindMarked()) != NULL)
	{
		menu_item->SetMarked(false);
		menu = menu_item->Submenu();
		
		if (menu == NULL)
		{
			continue;
		}
		
		while ((menu_item = menu->FindMarked()) != NULL)
		{
			menu_item->SetMarked(false);
		}
	}
	
	if (a_family == NULL
		|| a_style == NULL)
	{
		Init();
		return;
	}
	
	strcpy(m_family, a_family);
	strcpy(m_style, a_style);
	
	// Mark the new family and style
	menu_item = FindItem(m_family);
	
	if (menu_item != NULL)
	{
		menu_item->SetMarked(true);
		menu_item = menu_item->Submenu()->FindItem(m_style);
		
		if (menu_item != NULL)
		{
			menu_item->SetMarked(true);
		}
	}
}

//====================================================================
FontMenu::FontMenu(BRect a_frame, const char * const a_view_name, uint32 a_resize_mask, uint32 a_flags, menu_layout a_layout, bool a_resize_to_fit) :
//====================================================================
	BMenu(a_frame, a_view_name, a_resize_mask, a_flags, a_layout, a_resize_to_fit)
{
	Init();
}

//====================================================================
int FontMenu::MenuItemComp(BMenuItem * const * const a_1, BMenuItem * const * const a_2)
//====================================================================
{
	const char *s1, *s2;
	
	// Get the BMenus' names
	s1 = (*a_1)->Label();
	s2 = (*a_2)->Label();
	
	// Compare them
	return strcasecmp(s1, s2);
}

//====================================================================
BMenuItem *FontMenu::ConstructFamilyMenuItem(const font_family &a_family, const bool a_render_style, const bool a_render_family)
//====================================================================
{
	// Create the BMenu
	BMenu *family_menu;
	
	if ((family_menu = new BMenu(a_family)) == NULL)
	{
		return NULL;
	}
	
	// Create FontMenuItem for each style in the family
	int32 i, num_styles(count_font_styles(const_cast<font_family &>(a_family)));
	font_style style, def_style = "";
	uint32 flags;
	BMenuItem *style_item;
	
	for (i = 0; i < num_styles; i++)
	{
		// Get the style
		if (get_font_style(const_cast<font_family &>(a_family), i, &style, &flags) != B_NO_ERROR)
		{
			continue;
		}
		
		// Retain the default style
		if (def_style[0] == '\0')
		{
			strcpy(def_style, style);
		}
		
		// Create the FontMenuItem
		if ((style_item = new FontMenuItem(a_family, style, a_render_style)) == NULL)
		{
			break;
		}
		
		// Add the style FontMenuItem to the BMenu
		family_menu->AddItem(style_item);
	}
	
	// Check for a memory allocation error
	if (i < num_styles)
	{
		delete family_menu;
		return NULL;
	}
	
	// Make sure the submenu has the same properties
	BFont font;
	GetFont(&font);
	family_menu->SetFont(&font, ~B_FONT_FAMILY_AND_STYLE);
	
	// Create a BMenuItem for the family BMenu
	BMenuItem *family_item;
	
	if ((family_item = new FontMenuItem(family_menu, a_family, def_style, a_render_family)) == NULL)
	{
		delete family_menu;
		return NULL;
	}
	
	return family_item;
}

//====================================================================
void FontMenu::Init(void)
//====================================================================
{
	m_family[0] = '\0';
	m_style[0] = '\0';
}
